/*
**	DrawIt converter for 3Com PalmPilots
**	di_image.cpp
**
**	Copyright (C) 2001  William McFarland
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation; either version 2 of the License, or
**	(at your option) any later version.
** 
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
** 
**	You should have received a copy of the GNU General Public License
**	along with this program; if not, write to the Free Software
**	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

#include "di_image.h"
#include "clr_tbl.h"
#include <iostream>
#include <assert.h>

using namespace std;

// ctor for dynamic allocations
di_image::di_image (void)
    : img_num (0),
      offset (0),
      img_size (0),
      compressed (false),
      buff (NULL)
{
// Nothing to do
}

// ctor for static constructiosn
di_image::di_image(int num, unsigned int off, unsigned int sz, int cmp)
    :  img_num (num),
       offset (off),
       img_size (sz),
       compressed (cmp),
       buff(NULL)
{
// Nothing else to do
}

// dtor
di_image::~di_image (void)
{
    if (buff != NULL)
    {
        delete [] buff;
    }
}

// Read in the the image from the database.
size_t di_image::read_buffer (FILE * fin)
{
    // If the buffer exists we must delete it.
    if (buff !=  NULL)
    {
        delete [] buff;
        buff = NULL;
    }
    buff = new Byte [img_size];
    memset (buff, 0, img_size);

    // Get to the begining of the buffer
    fseek (fin, offset, SEEK_SET);
    fread (buff, img_size, 1, fin);

    // Now that we have the buffer, is it compressed?
    if (0xFE == buff[0] && 0xFD == buff[1])
    {
        compressed = true;
    }
    else
    {
        compressed = false;
    }
}

// Write the ppm image, the exact routine is determined by the
// the compressed flag.
void di_image::write_image (const char * baseName)
{
    char name [256];
    // Create the filename
    sprintf(name, "%s_%d.ppm", baseName, (img_num+1));
    // Open the file for writing
    FILE * fout = fopen (name, "w");
    // Call the correct writting method
    if (compressed)
    {
        write_compressed (fout);
    }
    else
    {
        write_uncompressed (fout);
    }
    fclose (fout);
    return;
}

// The routine to write a ppm image from a compressed image in the DrawIt Db.
// DrawIt uses a simple run encoding compression.
void di_image::write_compressed (FILE * fout)
{
    unsigned int count = 0;
    unsigned int color = 0;

    // Print out the header for the ppm file
    fprintf(fout, "P6\n%d %d\n%d\n", WIDTH, HEIGHT, MAX_COLORS);
    
    // For a compressed image the first two bytes are the compressed
    // signature, so we must start at byte 2.
    for (int cnt=2; cnt<img_size; )
    {
        // Get the count of the colors and the color to be repeated
        count = (unsigned int)buff[cnt++];
        color = (unsigned int)buff[cnt++];
        
        for (unsigned int idx=0; idx<count; idx++)
        {
            if (color > 255)
            {
                cout << "Color out of bounds\n";
            }
            
            // Print out the color table values
            fwrite(colormap[color], 1, 3, fout);
        }
    }
    return;
}

// This routine writes a ppm image from an uncompressed image in the DrawIt DB.
//
// Precondition img_size == 25600  the current size of an uncompressed image
//
void di_image::write_uncompressed (FILE * fout)
{
    assert (25600 == img_size);
    
    unsigned int color = 0;
    fprintf(fout, "P6\n%d %d\n%d\n", WIDTH, HEIGHT, MAX_COLORS);
    
    for (int cnt=0; cnt<img_size; cnt++)
    {
        color = (unsigned int)buff[cnt++];
        // Print out the color table values
        fwrite(colormap[color], 1, 3, fout);
    }
    return;
}

// The following are simple accessors
void di_image::set_image_number (const int num)
{
    img_num = num;
}

void di_image::set_offset (const long off)
{
    offset = off;
}

void di_image::set_size (const long sz)
{
    img_size = sz;
}

long di_image::get_size (void) const
{
    return img_size;
}

long di_image::get_offset (void) const
{
    return offset;
}
