/*
**  DrawIt converter for 3Com PalmPilots
**  drawit2ppm.h
**
**  Copyright (C) 2001  William McFarland
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
** 
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
** 
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <limits.h>
#include "di_image.h"

/*****************************************************************************
 *
 *  Define integral type Byte, Word, and DWord to match those on the
 *  Pilot being 8, 16, and 32 bits, respectively.
 *
 *****************************************************************************/

#define U8_MAX      255u        /* max  8-bit unsigned */
#if UCHAR_MAX == U8_MAX
typedef unsigned char   Byte;
#else
#error machine does not seem to support an 8-bit integral type
#endif

#define U16_MAX     65535u      /* max 16-bit unsigned */
#if USHRT_MAX == U16_MAX
typedef unsigned short  Word;
#else
#error machine does not seem to support a 16-bit integral type
#endif

#define U32_MAX     4294967295ul    /* max 32-bit unsigned */
#if USHRT_MAX == U32_MAX
typedef unsigned short  DWord;
#elif   UINT_MAX  == U32_MAX
typedef unsigned int    DWord;
#elif   ULONG_MAX == U32_MAX
typedef unsigned long   DWord;
#else
#error machine does not seem to support a 32-bit integral type
#endif

/*****************************************************************************
 *
 *  Globals functions
 *
 *****************************************************************************/
void        extract (char const *infile, int num, char const *basename);
void        extract_all (char const *infile, char const *basename);
int         load_data (char const *infile, di_image * &img);

Word        (*swap_Word )(  Word );
DWord       (*swap_DWord)( DWord );
void        select_swap();
Word        swap_Word_12( Word );
Word        swap_Word_21( Word );
DWord       swap_DWord_1234( DWord );
DWord       swap_DWord_4321( DWord );
void        usage(void);
FILE*       open_src_file ( char const *file_name );
FILE*       open_dest_file( char const *file_name );

#define NEW_BUFFER(b)   (b)->buf = malloc( (b)->len = BUFFER_SIZE )
#define GET_Word(f,n)   { fread( &n, 2, 1, f ); n = swap_Word ( n ); }
#define GET_DWord(f,n)  { fread( &n, 4, 1, f ); n = swap_DWord( n ); }
#define PUT_Word(f,n)   {  Word n1 = swap_Word ( n ); fwrite( &n1, 2, 1, f ); }
#define PUT_DWord(f,n)  { DWord n1 = swap_DWord( n ); fwrite( &n1, 4, 1, f ); }

/*****************************************************************************
 *
 *  Globals variables
 *
 *****************************************************************************/
char const* me;             /* executable name */
bool verbose = false;

#define VERSION     "0.2"
#define TYPE        "DATA"
#define CREATOR     "DRaw"
#define dmDBNameLength  32  /* 31 chars + 1 null terminator */
#define eol "\n"

const int PDB_HEADER_SIZE = 78;
const int PDB_RECORD_HEADER_SIZE = 8;
const int BUFF_SIZE = 500;

/*****************************************************************************
 *
 *  Global structures
 *
 *****************************************************************************/

typedef struct {        /* 78 bytes total */
    char    name[ dmDBNameLength ];
    Word    flags;
    Word    version;
    DWord   create_time;
    DWord   modify_time;
    DWord   backup_time;
    DWord   modificationNumber;
    DWord   appInfo_offset;
    DWord   sortInfo_offset;
    char    type[4];
    char    creator[4];                // Unique, must be DRaw
    DWord   id_seed;
    DWord   nextRecordList;
    Word    numRecords;               // How many images
} pdb_header;

typedef struct {    
    DWord   offset; 
    char    attributes;
    char    id[3];  
} drawit_record;
